
#include <string>
#include <thread>
#include "card_person.h"
#include "loc_point.h"
#include "card_message_handle.h"
#include "area.h"
#include "mine.h"
#include "his_location.h"
#include "three_rates.h"
#include "select_tool.h"
#include "monkey_car/monkeycar_person.h"
#include "websocket/ws_common.h"
#include "event.h"
#include "mine_business.h"
#include"common_tool.h"
#include"db/db_tool.h"
#include "forbid_staff_down_mine.h"

person::person(const std::string &type,uint32_t cardid,uint16_t needdisplay,int16_t t,int32_t deptid,int32_t level_id,uint32_t cid,int wl,const std::string &sname,const std::string &dname,int worketype_id)
	:card_location_base(type,cardid,needdisplay,t,deptid,level_id,cid)
	,m_workLine(wl)
	,m_stafferName(sname)
	,m_deptName(dname)
	,m_worktype_id(worketype_id)
{
	m_message_handle.reset(new card_message_handle(this));
	m_his_location_card.reset(new location_staff(m_id,m_type,cid));
}

person::~person()
{
}
void person::clear()
{
	m_area_tool.reset(new area_tool);
	m_mine_tool.reset(new mine_tool);
	card_location_base::clear();
    uint64_t id = tool_other::type_id_to_u64(m_type,m_id);
    for(std::uint8_t i=0;i<CARD_EVENT_COUNT_MAX;i++)
    {
        if(m_event[i]!=0)
        {
            event_tool::instance()->handle_event(OT_CARD,static_cast<EVENT_TYPE>(i),id,0,0,true);
            m_event[i]=0;
        }
    }
}

void person::set_area_info(int mapid,double scale,int areaid,uint64_t t,int type)
{
	m_area_tool->set_area_info(mapid,scale,areaid,*this,t,type);
}
void person::site_hover(int sid)
{
	IKSDK_DB(sid);
}
//虹膜识别入库功能	
void person::IKSDK_DB(int sid)
{
	time_t now = time(0);
	if(now - m_iris_recognition_timeval > 5)
	{
		if (forbid_staff_down_mine::instance()->IsForbid(m_cid,now))
		{
			log_info("Staff:%d forbid down mine",m_cid); //禁止指定人员下井
			return;
		}
		int f=0;
		if(event_list::instance()->get_event_card(m_id, m_type, ET_CARD_LOW_POWER_SERIOUS)) {
			f = 1;
		}
		std::string card_id = tool_other::type_id_to_str(m_type,m_id);
		std::string st	    = tool_time::to_str(now);
		char nsql[256]={0};
		const char*sql="REPLACE INTO his_att_interface (staff_id,card_id,reader_id,staff_name,dept_name,upt_time,low_power_warn) VALUES (%d,%s,%d,'%s','%s','%s',%d);";
		snprintf(nsql,256,sql,m_cid,card_id.c_str(),sid,m_stafferName.c_str(),m_deptName.c_str(),st.c_str(),f);
		db_tool::PushAsync(nsql);
		logn_info(2,"%s",nsql);
		m_iris_recognition_timeval = now;
	}
}

std::shared_ptr<mine_tool> person::get_mine_tool()
{
	return m_mine_tool;
}

void person::do_business(const std::shared_ptr<site>&site,const point &pt,double acc)
{
    m_area_tool->set(site);
	m_area_tool->on_point(shared_from_this(),pt);
	m_timeval=m_time;
	handle_three_rates(pt);
}

void person::reset(std::shared_ptr<monkey_person> mp)
{
	m_monkeyPerson = mp;
}

void person::handle_three_rates(const point & pt)
{
	card_pos cp;
	cp.work_line=m_workLine;
	cp.work_type_id = m_worktype_id;
	m_biz_stat=get_stat();
	cp.biz_stat = m_biz_stat;
	cp.x=pt.x;cp.y=pt.y;cp.z=pt.z;
	cp.work_type_id=m_worktype_id;
	put_three_rates(cp);
}

void person::on_timer()
{
    if(!m_mine_tool->m_is_attendance)
		return;
	YA::_CARD_POS_ cp;
	uint64_t _time=0;
	point pt = getSmoothPoint(_time);
    
	cp.area_info=m_area_tool->m_area_info;	
	cp.map_id =m_area_tool->m_mapid;

	cp.biz_stat = m_biz_stat;
	cp.down_time = m_mine_tool->get_down_time();
	cp.work_time = m_mine_tool->get_work_time();
	cp.is_on_duty= m_mine_tool->is_on_duty();
	upt_card_pos(cp,pt);
	log_info("on_timer here ...%d,%lld,%.2f,%.2f,%d,%d--",m_id,_time,pt.x,pt.y,cp.map_id,cp.area_info.size());
	if(!cp.area_info.empty())
	{
		int area_id = cp.area_info.begin()->first;
		m_his_location_card->push(_time,pt,area_id,cp.map_id);
	}
	uint64_t _now=tool_time::now_to_ms();
	uint64_t t=_now>m_timeval?_now-m_timeval:m_timeval-_now;
	if(t>10*1000)
	{
		m_area_tool->on_point(shared_from_this(),pt);
		m_biz_stat=get_stat();
	}
}

point person::getSmoothPoint(uint64_t& t)
{
	point pt;
	loc_point lp = m_smo_tool->smooth_strategy();
	m_speed = lp.m_speed;
	m_stat  = lp.m_stat;
	pt.x = lp.x;
	pt.y = -lp.y;
	t=lp.m_time;
	if(auto  p = m_monkeyPerson.lock() )
	{
		if(p->is_on_bus())
		{
			m_stat = 7;
			pt = p->getPoint(t);	
			log_info("getpoint_oncar:%d,%lld",m_id,t);
		}
	}
	return pt;
}
void person::get_card(bool f)
{
    if(!m_mine_tool->m_is_attendance)
		return;
	mine_business::inst()->fetch_add(m_display);
}

