#include "event.h"
#include "area.h"
#include "card_base.h"
#include "common_tool.h"
#include "log.h"

#include "area_business_count_checker.h"
/*
	判断当前区域a中的人数是否超过设定人数，超过后告警
	区域内实时人数存储在area对象中,在当前类on_enter/on_leave中进行更新
	整个井下的超员和某个区域的超员都使用这个代码
*/

static void check_event(bool &event_flag,EVENT_TYPE et,EVENT_DIS_TYPE edt,int id)
{
    if(!event_flag)
    {
        auto ev_p = event_list::instance()->get_event_area(id,et,edt);
        if (ev_p && !ev_p->is_end())
            event_flag = true;
    }
}

//服务器重启加载数据
void area_business_count_checker::on_load_his(const std::shared_ptr<area_hover>&a,const std::shared_ptr<card_location_base>&c,std::shared_ptr<business_data>&ptr)
{
    if (nullptr == a->m_area )
        return ;
    if (c->is_person())
    {
        EVENT_TYPE ev = a->m_area->is_mine()?EVENT_TYPE::ET_OVER_COUNT_PERSON : EVENT_TYPE::ET_AREA_OVER_COUNT_PERSON ;
        check_event(a->m_area->m_event_person_count,ev,DT_NORMAL,a->m_area->id());
        check_event(a->m_area->m_event_person_show_count,ev,DT_SPECIAL,a->m_area->id());
    }
    else if (c->is_vehicle())
    {
        EVENT_TYPE ev = a->m_area->is_mine()?EVENT_TYPE::ET_OVER_COUNT_VEHICLE : EVENT_TYPE::ET_AREA_OVER_COUNT_VEHICLE ;
        check_event(a->m_area->m_event_vehicle_count,ev,DT_NORMAL,a->m_area->id());
        check_event(a->m_area->m_event_vehicle_show_count,ev,DT_SPECIAL,a->m_area->id());
    }
    on_enter(a,c,ptr);
}
//增加计数,并进行判断
void area_business_count_checker::on_enter(const std::shared_ptr<area_hover>&a,
                                           const std::shared_ptr<card_location_base>&c,std::shared_ptr<business_data>&ptr)
{
	if (nullptr == a->m_area )
		return ;
	if (c->is_person())
	{
		a->m_area->m_person_count ++ ;
        if(c->m_display)
		    a->m_area->m_person_show_count ++ ;
        int pc=a->m_area->m_person_count.load();
        int pc_=a->m_area->m_person_show_count.load();
        int limit_val=a->m_area->m_limit_person_count;
        int aid=a->m_area->id();
	    EVENT_TYPE ev = a->m_area->is_mine()?EVENT_TYPE::ET_OVER_COUNT_PERSON : EVENT_TYPE::ET_AREA_OVER_COUNT_PERSON ;
        if(pc>limit_val){
		    event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,pc,a->m_area->m_event_person_count=true,DT_NORMAL);
		    log_info("person_count_enter:%d,v_count:%d limit:%d",aid,pc,limit_val);
        }
        if(pc_>limit_val){
		    event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,pc_,a->m_area->m_event_person_show_count=true,DT_SPECIAL);
		    log_info("person_count_enter_show:%d,v_count:%d limit:%d",aid,pc,limit_val);
        }
	}
	else if (c->is_vehicle())
	{
		a->m_area->m_vehicle_count ++ ;
        if(c->m_display)
		    a->m_area->m_vehicle_show_count ++ ;
        int vc=a->m_area->m_vehicle_count.load();
        int vc_=a->m_area->m_vehicle_show_count.load();
        int limit_val=a->m_area->m_limit_vehicle_count;
        int aid=a->m_area->id();
		EVENT_TYPE ev = a->m_area->is_mine()?EVENT_TYPE::ET_OVER_COUNT_VEHICLE : EVENT_TYPE::ET_AREA_OVER_COUNT_VEHICLE ;
        if(vc>limit_val){
		    event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,vc,a->m_area->m_event_vehicle_count=true,DT_NORMAL);
		    log_info("vehicle_count_enter:%d,v_count:%d limit:%d",aid,vc,limit_val);
        }
        if(vc_>limit_val){
		    event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,vc_,a->m_area->m_event_vehicle_show_count=true,DT_SPECIAL);
		    log_info("vehicle_count_enter_show:%d,v_count:%d limit:%d",aid,vc,limit_val);
        }
	}
}
void area_business_count_checker::on_hover(const std::shared_ptr<area_hover>&a,
                                           const std::shared_ptr<card_location_base>&c,std::shared_ptr<business_data> ptr)
{
}

//减少计数
void area_business_count_checker::on_leave(const std::shared_ptr<area_hover>&a,
                                           const std::shared_ptr<card_location_base>&c,std::shared_ptr<business_data> ptr)
{
	if (nullptr == a->m_area )
		return ;
	if (c->is_person())
	{
		a->m_area->m_person_count -- ;
        if(c->m_display)
		    a->m_area->m_person_show_count -- ;
        int pc=a->m_area->m_person_count.load();
        int pc_=a->m_area->m_person_show_count.load();
        int limit_val=a->m_area->m_limit_person_count;
        int aid=a->m_area->id();
	    EVENT_TYPE ev = a->m_area->is_mine()?EVENT_TYPE::ET_OVER_COUNT_PERSON : EVENT_TYPE::ET_AREA_OVER_COUNT_PERSON ;
		if(a->m_area->m_event_person_count&& pc <= limit_val){
			event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,pc,a->m_area->m_event_person_count=false,DT_NORMAL);
		    log_info("person_count_leave:%d,v_count:%d limit:%d",aid,pc,limit_val);
        }
		if(a->m_area->m_event_person_show_count&& pc_ <= limit_val){
			event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,pc_,a->m_area->m_event_person_show_count=false,DT_SPECIAL);
		    log_info("person_count_leave_show:%d,v_count:%d limit:%d",aid,pc,limit_val);
        }
	}
	else if (c->is_vehicle())
	{
        a->m_area->m_vehicle_count -- ;
        if(c->m_display)
		    a->m_area->m_vehicle_show_count -- ;
        int pc=a->m_area->m_vehicle_count.load();
        int pc_=a->m_area->m_vehicle_show_count.load();
        int limit_val=a->m_area->m_limit_vehicle_count;
        int aid=a->m_area->id();
		EVENT_TYPE ev = a->m_area->is_mine()?EVENT_TYPE::ET_OVER_COUNT_VEHICLE : EVENT_TYPE::ET_AREA_OVER_COUNT_VEHICLE ;
		if(a->m_area->m_event_vehicle_count&& pc <= limit_val){
			event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,pc,a->m_area->m_event_vehicle_count=false,DT_NORMAL);
		    log_info("vehicle_count_leave:%d,v_count:%d limit:%d",aid,pc,limit_val);
        }
		if(a->m_area->m_event_vehicle_show_count&& pc_ <= limit_val){
			event_tool::instance()->handle_event(OT_AREA,ev,aid,limit_val,pc_,a->m_area->m_event_vehicle_show_count=false,DT_SPECIAL);
		    log_info("vehicle_count_leave_show:%d,v_count:%d limit:%d",aid,pc,limit_val);
        }
	}
}

